@@ -39,9 +39,22 @@ if [ -z "${CI_CONFIG_QUIET-}" ] || [ "${CI_CONFIG_QUIET-}" = yes ] || [ "${CI_CO
 fi
 
 pushd ../../..
+rm -rf tmp-deps
+mkdir -p tmp-deps
 
 # Clone and build dependencies
 [ -z "$CI_TIME" ] || echo "`date`: Starting build of dependencies (if any)..."
+BASE_PWD=${PWD}
+cd tmp-deps
+wget http://ftp.gnu.org/gnu/libmicrohttpd/libmicrohttpd-0.9.44.tar.gz
+tar -xzf $(basename "http://ftp.gnu.org/gnu/libmicrohttpd/libmicrohttpd-0.9.44.tar.gz")
+cd $(basename "http://ftp.gnu.org/gnu/libmicrohttpd/libmicrohttpd-0.9.44.tar.gz" .tar.gz)
+$CI_TIME ./configure "${CONFIG_OPTS[@]}"
+$CI_TIME make -j4
+$CI_TIME make install
+cd ${BASE_PWD}
+BASE_PWD=${PWD}
+cd tmp-deps
 $CI_TIME git clone --quiet --depth 1 https://github.com/zeromq/libzmq.git libzmq
 cd libzmq
 git --no-pager log --oneline -n1
@@ -62,7 +75,31 @@ fi
 $CI_TIME ./configure "${CONFIG_OPTS[@]}"
 $CI_TIME make -j4
 $CI_TIME make install
-cd ..
+cd ${BASE_PWD}
+
+BASE_PWD=${PWD}
+cd tmp-deps
+$CI_TIME git clone --quiet --depth 1 https://github.com/curl/curl.git libcurl
+cd libcurl
+git --no-pager log --oneline -n1
+if [ -e autogen.sh ]; then
+    $CI_TIME ./autogen.sh 2> /dev/null
+fi
+if [ -e buildconf ]; then
+    $CI_TIME ./buildconf 2> /dev/null
+fi
+if [ ! -e autogen.sh ] && [ ! -e buildconf ] && [ ! -e ./configure ] && [ -s ./configure.ac ]; then
+    $CI_TIME libtoolize --copy --force && \
+    $CI_TIME aclocal -I . && \
+    $CI_TIME autoheader && \
+    $CI_TIME automake --add-missing --copy && \
+    $CI_TIME autoconf || \
+    $CI_TIME autoreconf -fiv
+fi
+$CI_TIME ./configure "${CONFIG_OPTS[@]}"
+$CI_TIME make -j4
+$CI_TIME make install
+cd ${BASE_PWD}
 
 popd
 pushd ../..
