@@ -40,6 +40,8 @@ struct _zframe_t {
 #ifdef ZMQ_GROUP_MAX_LENGTH
     char group[ZMQ_GROUP_MAX_LENGTH + 1];
 #endif
+    zframe_destructor_fn *destructor;   //  Destructor for memory
+    void * hint;                        //  Hint for destroying the memory
 };
 
 //  --------------------------------------------------------------------------
@@ -96,6 +98,7 @@ zframe_destroy (zframe_t **self_p)
         zmq_msg_close (&self->zmsg);
         self->tag = 0xDeadBeef;
         freen (self);
+
         *self_p = NULL;
     }
 }
@@ -111,6 +114,47 @@ zframe_from (const char *string)
     return zframe_new (string, strlen (string));
 }
 
+//  --------------------------------------------------------------------------
+//  Create a new frame from memory. Taking ownership of the memory and calling the destructor
+//  on destroy.
+
+struct zmq_msg_free_arg {
+    zframe_destructor_fn *destructor;
+    void *hint;
+};
+
+static void
+zmq_msg_free (void *data, void *hint) {
+    struct zmq_msg_free_arg *arg = (struct zmq_msg_free_arg *)hint;
+    arg->destructor (&arg->hint);
+
+    freen (arg);
+}
+
+zframe_t *
+zframe_frommem (void *data, size_t size, zframe_destructor_fn destructor, void *hint) {
+    assert (data);
+
+    zframe_t *self = (zframe_t *) zmalloc (sizeof (zframe_t));
+    assert (self);
+    self->tag = ZFRAME_TAG;
+    self->destructor = destructor;
+    self->hint = hint;
+
+    struct zmq_msg_free_arg *arg = (struct zmq_msg_free_arg*) zmalloc (sizeof (struct zmq_msg_free_arg));
+    arg->destructor = destructor;
+    arg->hint = hint;
+
+    //  Catch heap exhaustion in this specific case
+    if (zmq_msg_init_data (&self->zmsg, data, size, zmq_msg_free, arg)) {
+        freen (arg);
+        zframe_destroy (&self);
+        return NULL;
+    }
+
+    return self;
+}
+
 
 //  --------------------------------------------------------------------------
 //  Receive frame from socket, returns zframe_t object or NULL if the recv
@@ -578,6 +622,12 @@ zframe_fprint (zframe_t *self, const char *prefix, FILE *file)
 //  --------------------------------------------------------------------------
 //  Selftest
 
+static void
+mem_destructor (void **hint) {
+    strcpy ((char*)*hint, "world");
+}
+
+
 void
 zframe_test (bool verbose)
 {
@@ -586,7 +636,7 @@ zframe_test (bool verbose)
     zframe_t* frame;
 
     //  @selftest
-    //  Create two PAIR sockets and connect over inproc
+    //  Create two PAIR sockets and connect over TCP
     zsock_t *output = zsock_new (ZMQ_PAIR);
     assert (output);
     int port = zsock_bind (output, "tcp://127.0.0.1:*");
@@ -753,6 +803,15 @@ zframe_test (bool verbose)
     zframe_destroy (&frame);
 #endif
 
+    char str[] = "hello";
+    frame = zframe_frommem (str, 5, mem_destructor, str);
+    assert (frame);
+    zframe_destroy (&frame);
+
+    //  The destructor doesn't free the memory, only changing the strid,
+    //  so we can check if the destructor was invoked
+    assert (streq (str, "world"));
+
 #if defined (__WINDOWS__)
     zsys_shutdown();
 #endif
