@@ -82,6 +82,7 @@ static bool s_initialized = false;
 static size_t s_io_threads = 1;     //  ZSYS_IO_THREADS=1
 static int s_thread_sched_policy = -1; //  ZSYS_THREAD_SCHED_POLICY=-1
 static int s_thread_priority = -1;  //  ZSYS_THREAD_PRIORITY=-1
+static int s_thread_name_prefix = -1;  //  ZSYS_THREAD_NAME_PREFIX=-1
 static size_t s_max_sockets = 1024; //  ZSYS_MAX_SOCKETS=1024
 static int s_max_msgsz = INT_MAX;   //  ZSYS_MAX_MSGSZ=INT_MAX
 static int64_t s_file_stable_age_msec = S_DEFAULT_ZSYS_FILE_STABLE_AGE_MSEC;
@@ -133,6 +134,11 @@ typedef CRITICAL_SECTION zsys_mutex_t;
 //  Mutex to guard socket counter
 static zsys_mutex_t s_mutex;
 
+//  Implementation for the zsys_vprintf() which is known from legacy
+//  and poses as a stable interface now.
+static inline
+char *
+s_zsys_vprintf_hint (int hint, const char *format, va_list argptr);
 
 //  --------------------------------------------------------------------------
 //  Initialize CZMQ zsys layer; this happens automatically when you create
@@ -258,6 +264,11 @@ zsys_init (void)
     else
         zsys_set_thread_sched_policy (s_thread_sched_policy);
 
+    if (getenv ("ZSYS_THREAD_NAME_PREFIX"))
+        zsys_set_thread_name_prefix (atoi (getenv ("ZSYS_THREAD_NAME_PREFIX")));
+    else
+        zsys_set_thread_name_prefix (s_thread_name_prefix);
+
     return s_process_ctx;
 }
 
@@ -312,6 +323,7 @@ zsys_shutdown (void)
       s_io_threads = 1;
       s_thread_sched_policy = -1;
       s_thread_priority = -1;
+      s_thread_name_prefix = -1;
       s_max_sockets = 1024;
       s_max_msgsz = INT_MAX;
       s_file_stable_age_msec = S_DEFAULT_ZSYS_FILE_STABLE_AGE_MSEC;
@@ -907,6 +919,23 @@ zsys_version (int *major, int *minor, int *patch)
 }
 
 
+//  --------------------------------------------------------------------------
+//  Format a string using printf formatting, returning a freshly allocated
+//  buffer. If there was insufficient memory, returns NULL. Free the returned
+//  string using zstr_free(). The hinted version allows to optimize by using
+//  a larger starting buffer size (known to/assumed by the developer) and so
+//  avoid reallocations.
+
+char *
+zsys_sprintf_hint (int hint, const char *format, ...)
+{
+    va_list argptr;
+    va_start (argptr, format);
+    char *string = s_zsys_vprintf_hint (hint, format, argptr);
+    va_end (argptr);
+    return (string);
+}
+
 //  --------------------------------------------------------------------------
 //  Format a string using printf formatting, returning a freshly allocated
 //  buffer. If there was insufficient memory, returns NULL. Free the returned
@@ -915,23 +944,32 @@ zsys_version (int *major, int *minor, int *patch)
 char *
 zsys_sprintf (const char *format, ...)
 {
+    // Effectively this is a copy of the small zsys_sprintf_hint with
+    // hardcoded hint value; this is is cheaper than parsing va_list
+    // several times to just call the other implementation cleanly.
     va_list argptr;
     va_start (argptr, format);
-    char *string = zsys_vprintf (format, argptr);
+    char *string = s_zsys_vprintf_hint (256, format, argptr);
     va_end (argptr);
     return (string);
 }
 
-
 //  --------------------------------------------------------------------------
 //  Format a string with variable arguments, returning a freshly allocated
 //  buffer. If there was insufficient memory, returns NULL. Free the returned
 //  string using zstr_free().
 
+static inline
 char *
-zsys_vprintf (const char *format, va_list argptr)
+s_zsys_vprintf_hint (int hint, const char *format, va_list argptr)
 {
-    int size = 256;
+    if (hint <= 0) {
+        //  The hint is not a hard requrement so no error here.
+        //  Just fall back to legacy default.
+        hint = 256;
+    }
+    // Must use int "size" to compare to "required" (from vsnprintf)
+    int size = hint;
     char *string = (char *) malloc (size);
     if (!string)
         return NULL;
@@ -953,9 +991,18 @@ zsys_vprintf (const char *format, va_list argptr)
         va_end (my_argptr);
     }
 #endif
+    if (required < 0) {
+        // vsnprintf failed at unknown point; at least prohibit
+        // accesses by string consumers into random memory
+        string [hint - 1] = '\0';
+        return string;
+    }
     //  If formatted string cannot fit into small string, reallocate a
-    //  larger buffer for it.
-    if (required >= size) {
+    //  larger buffer for it. If it did fit, we only called vsnprintf()
+    //  once and already have the good result, so pre-counting with a
+    //  NULL string would not be beneficial for shorter texts (always
+    //  calling vsnprintf() twice then).
+    if (required >= hint) {
         size = required + 1;
         freen (string);
         string = (char *) malloc (size);
@@ -968,6 +1015,11 @@ zsys_vprintf (const char *format, va_list argptr)
     return string;
 }
 
+char *
+zsys_vprintf (const char *format, va_list argptr)
+{
+    return (s_zsys_vprintf_hint(256, format, argptr));
+}
 
 //  --------------------------------------------------------------------------
 //  Create a UDP beacon socket; if the routable option is true, uses
@@ -980,14 +1032,27 @@ zsys_udp_new (bool routable)
     //  We haven't implemented multicast yet
     assert (!routable);
     SOCKET udpsock;
+    int type = SOCK_DGRAM;
+#ifdef CZMQ_HAVE_SOCK_CLOEXEC
+    //  Ensure socket is closed by exec() functions.
+    type |= SOCK_CLOEXEC;
+#endif
+
     if (zsys_ipv6 ())
-        udpsock = socket (AF_INET6, SOCK_DGRAM, IPPROTO_UDP);
+        udpsock = socket (AF_INET6, type, IPPROTO_UDP);
     else
-        udpsock = socket (AF_INET, SOCK_DGRAM, IPPROTO_UDP);
+        udpsock = socket (AF_INET, type, IPPROTO_UDP);
     if (udpsock == INVALID_SOCKET) {
         zsys_socket_error ("socket");
         return INVALID_SOCKET;
     }
+
+    //  On Windows, preventing sockets to be inherited by child processes.
+#if defined (__WINDOWS__) && defined (HANDLE_FLAG_INHERIT)
+    if (!SetHandleInformation ((HANDLE) udpsock, HANDLE_FLAG_INHERIT, 0))
+        zsys_socket_error ("SetHandleInformation (HANDLE_FLAG_INHERIT)");
+#endif
+
     //  Ask operating system for broadcast permissions on socket
     int on = 1;
     if (setsockopt (udpsock, SOL_SOCKET, SO_BROADCAST,
@@ -1403,6 +1468,98 @@ zsys_set_thread_sched_policy (int policy)
 }
 
 
+//  --------------------------------------------------------------------------
+//  Configure the numeric prefix to each thread created for the internal
+//  context's thread pool. This option is only supported on Linux.
+//  If the environment variable ZSYS_THREAD_NAME_PREFIX is defined, that
+//  provides the default.
+//  Note that this method is valid only before any socket is created.
+
+void
+zsys_set_thread_name_prefix (int prefix)
+{
+    if (prefix < 0)
+        return;
+
+    zsys_init ();
+    ZMUTEX_LOCK (s_mutex);
+    //  If the app is misusing this method, burn it with fire
+    if (s_open_sockets)
+        zsys_error ("zsys_set_thread_name_prefix() is not valid after"
+                " creating sockets");
+    assert (s_open_sockets == 0);
+    s_thread_name_prefix = prefix;
+#if defined (ZMQ_THREAD_NAME_PREFIX)
+    zmq_ctx_set (s_process_ctx, ZMQ_THREAD_NAME_PREFIX, s_thread_name_prefix);
+#endif
+    ZMUTEX_UNLOCK (s_mutex);
+}
+
+//  --------------------------------------------------------------------------
+//  Return ZMQ_THREAD_NAME_PREFIX option.
+int
+zsys_thread_name_prefix ()
+{
+    zsys_init ();
+    ZMUTEX_LOCK (s_mutex);
+#if defined (ZMQ_THREAD_NAME_PREFIX)
+    s_thread_name_prefix = zmq_ctx_get (s_process_ctx, ZMQ_THREAD_NAME_PREFIX);
+#endif
+    ZMUTEX_UNLOCK (s_mutex);
+    return s_thread_name_prefix;
+}
+
+
+//  --------------------------------------------------------------------------
+//  Adds a specific CPU to the affinity list of the ZMQ context thread pool.
+//  This option is only supported on Linux.
+//  Note that this method is valid only before any socket is created.
+
+void
+zsys_thread_affinity_cpu_add (int cpu)
+{
+    if (cpu < 0)
+        return;
+
+    zsys_init ();
+    ZMUTEX_LOCK (s_mutex);
+    //  If the app is misusing this method, burn it with fire
+    if (s_open_sockets)
+        zsys_error ("zsys_set_thread_sched_policy() is not valid after"
+                " creating sockets");
+    assert (s_open_sockets == 0);
+#if defined (ZMQ_THREAD_AFFINITY_CPU_ADD)
+    zmq_ctx_set (s_process_ctx, ZMQ_THREAD_AFFINITY_CPU_ADD, cpu);
+#endif
+    ZMUTEX_UNLOCK (s_mutex);
+}
+
+
+//  --------------------------------------------------------------------------
+//  Removes a specific CPU to the affinity list of the ZMQ context thread pool.
+//  This option is only supported on Linux.
+//  Note that this method is valid only before any socket is created.
+
+void
+zsys_thread_affinity_cpu_remove (int cpu)
+{
+    if (cpu < 0)
+        return;
+
+    zsys_init ();
+    ZMUTEX_LOCK (s_mutex);
+    //  If the app is misusing this method, burn it with fire
+    if (s_open_sockets)
+        zsys_error ("zsys_set_thread_sched_policy() is not valid after"
+                " creating sockets");
+    assert (s_open_sockets == 0);
+#if defined (ZMQ_THREAD_AFFINITY_CPU_REMOVE)
+    zmq_ctx_set (s_process_ctx, ZMQ_THREAD_AFFINITY_CPU_REMOVE, cpu);
+#endif
+    ZMUTEX_UNLOCK (s_mutex);
+}
+
+
 //  --------------------------------------------------------------------------
 //  Configure the scheduling priority of the ZMQ context thread pool.
 //  Not available on Windows. See the sched_setscheduler man page or sched.h
@@ -1826,6 +1983,185 @@ zsys_auto_use_fd (void)
     return s_auto_use_fd;
 }
 
+typedef enum _zsprintf_s {
+    FIND_PERCENT,
+    FIND_KEY,
+    FIND_FORMAT,
+    END
+} zsprintf_s;
+
+typedef void* (*zsys_lookup_fn)(void *, const char*);
+
+static char *
+s_zsys_zprintf (const char *format, void *args, zsys_lookup_fn lookup_fn, bool validate)
+{
+    assert (format);
+    assert (args);
+    zchunk_t *chunk = zchunk_new (NULL, strlen (format) * 1.5);
+    assert (chunk);
+    char *ret = NULL;
+
+    zsprintf_s state = FIND_PERCENT;
+    size_t pos = 0;
+    char *key = NULL;
+
+    while (state != END)
+    {
+
+        if (pos >= strlen (format))
+            break;
+
+        switch (state) {
+            case FIND_PERCENT:
+                {
+
+                    //zsys_debug ("\tstate=FIND_PERCENT, format+%zu=%s", pos, format+pos);
+                    char *percent = strchr ((char*)(format) + pos, '%');
+
+                    if (!percent) {
+                        //zsys_debug ("!percent");
+                        zchunk_extend (chunk, (format+pos), strlen (format) - pos);
+                        state = END;
+                    }
+                    else
+                    if (*(percent+1) == '%') {
+                        size_t idx = percent - format;
+                        //zsys_debug ("*(percent+1)=='%%':\tidx=%zu, format+%zu=%s", idx, pos, format+pos);
+                        if (idx - pos > 0) {
+                            //zsys_debug ("*(percent+1)=='%%':\t#2pos=%zu, idx-pos=%zu", pos, idx-pos);
+                            zchunk_extend (chunk, format+pos, idx - pos);
+                            pos += (idx-pos);
+                            //zsys_debug ("*(percent+1)=='%%':\t#2pos=%zu, idx-pos=%zu", pos, idx-pos);
+                        }
+                        zchunk_extend (chunk, "%", 1);
+                        pos += 2;
+                    }
+                    else
+                    if (*(percent+1) == '(') {
+                        size_t idx = percent - format;
+                        //zsys_debug ("*(percent+1) == '(': idx=%zu, pos=%zu", idx, pos);
+                        if (idx - pos > 0) {
+                            zchunk_extend (chunk, (format+pos), idx - pos);
+                            pos += (idx-pos);
+                        }
+                        //zsys_debug ("*(percent+1) == '(': idx=%zu, pos=%zu", idx, pos);
+                        state = FIND_KEY;
+                    }
+                    else {
+                        //zsys_debug ("else");
+                        size_t idx = percent - format;
+                        zchunk_extend (chunk, (format+pos), idx - pos);
+                        pos += (idx-pos);
+                    }
+                }
+                break;
+            case FIND_KEY:
+                {
+                    //zsys_debug ("\tstate=FIND_KEY, format+%zu=%s", pos, format+pos);
+                    char *key_end = strchr ((char*)(format)+pos, ')');
+                    if (!key_end) {
+                        zchunk_extend (chunk, (format+pos), strlen (format) - pos);
+                        state = END;
+                    }
+                    pos += 2;
+                    size_t idx = key_end - format;
+                    size_t key_len = idx - pos;
+                    if (key_len == 0) {
+                        zchunk_extend (chunk, "()", 2);
+                        pos += 2;
+                        state = FIND_PERCENT;
+                    }
+                    zstr_free (&key);
+                    key = (char*) zmalloc (key_len + 1);
+                    memcpy ((void*) key, format+pos, key_len);
+
+                    if (! lookup_fn (args, key)) {
+                        char *ret = NULL;
+                        if (validate)
+                            ret = zsys_sprintf ("Key '%s' not found in hash", key);
+                        zstr_free (&key);
+                        zchunk_destroy (&chunk);
+                        return ret;
+                    }
+                    pos += key_len + 1;
+                    state = FIND_FORMAT;
+                }
+                break;
+            case FIND_FORMAT:
+                {
+                    //zsys_debug ("\tstate=FIND_FORMAT, format+%zu=%s", pos, format+pos);
+                    if (*(format+pos) != 's') {
+                        char *ret = NULL;
+                        if (validate)
+                            ret = zsys_sprintf ("%s: arguments other than 's' are not implemented", key);
+                        zstr_free (&key);
+                        zchunk_destroy (&chunk);
+                        return ret;
+                    }
+                    pos += 1;
+                    char *v = (char *) lookup_fn (args, key);
+                    zchunk_extend (chunk, v, strlen (v));
+                    state = FIND_PERCENT;
+                }
+                break;
+            case END:
+                break;
+        }
+    }
+    zstr_free (&key);
+
+    //FIXME: is it needed?
+    zchunk_extend (chunk, "\0", 1);
+
+    if (!validate) {
+        ret = strdup ((char*) zchunk_data (chunk));
+        zchunk_destroy (&chunk);
+        return ret;
+    }
+
+    zchunk_destroy (&chunk);
+    return NULL;
+}
+
+
+// printf based on zhash_t
+char *
+zsys_zprintf (const char *format, zhash_t *args)
+{
+    return s_zsys_zprintf (format, (void*) args, (zsys_lookup_fn) zhash_lookup, false);
+}
+
+// return missing key or other format errors as new allocated string or NULL
+char *
+zsys_zprintf_error (const char *format, zhash_t *args)
+{
+    return s_zsys_zprintf (format, (void*) args, (zsys_lookup_fn) zhash_lookup, true);
+}
+
+static void *
+s_zconfig_lookup (void *container, const char *key)
+{
+    zconfig_t *root = (zconfig_t*) container;
+    zconfig_t *child = zconfig_locate (root, key);
+    if (child)
+        return zconfig_value (child);
+    return NULL;
+}
+
+// printf based on zconfig
+char *
+zsys_zplprintf (const char *format, zconfig_t *args)
+{
+    return s_zsys_zprintf (format, (void*) args, (zsys_lookup_fn) s_zconfig_lookup, false);
+}
+
+// return missing key or other format errors as new allocated string or NULL
+char *
+zsys_zplprintf_error (const char *format, zconfig_t *args)
+{
+    return s_zsys_zprintf (format, (void*) args, (zsys_lookup_fn) s_zconfig_lookup, true);
+}
+
 
 //  --------------------------------------------------------------------------
 //  Set log identity, which is a string that prefixes all log messages sent
@@ -2094,6 +2430,10 @@ zsys_test (bool verbose)
     zsys_set_ipv6 (0);
     zsys_set_thread_priority (-1);
     zsys_set_thread_sched_policy (-1);
+    zsys_set_thread_name_prefix (0);
+    assert (0 == zsys_thread_name_prefix());
+    zsys_thread_affinity_cpu_add (0);
+    zsys_thread_affinity_cpu_remove (0);
     zsys_set_zero_copy_recv(0);
     assert (0 == zsys_zero_copy_recv());
     zsys_set_zero_copy_recv(1);
@@ -2273,6 +2613,54 @@ zsys_test (bool verbose)
         zstr_free (&received);
     }
     zsys_close (logger, NULL, 0);
+
+    {
+        // zhash based printf
+        zhash_t *args = zhash_new ();
+        zhash_insert (args, "key", "value");
+        zhash_insert (args, "ham", "spam");
+
+        char *str = zsys_zprintf ("plain string", args);
+        assert (streq (str, "plain string"));
+        zstr_free (&str);
+
+        str = zsys_zprintf ("%%a%%", args);
+        assert (streq (str, "%a%"));
+        zstr_free (&str);
+
+        str = zsys_zprintf ("VALUE=%(key)s123", args);
+        assert (streq (str, "VALUE=value123"));
+        zstr_free (&str);
+
+        str = zsys_zprintf ("VALUE=%(key)s123, %(ham)s, %(ham)s, %%(nospam)s!!!", args);
+        assert (streq (str, "VALUE=value123, spam, spam, %(nospam)s!!!"));
+        zstr_free (&str);
+
+        str = zsys_zprintf ("VALUE=%(nokey)s123, %(ham)s, %(ham)s, %%(nospam)s!!!", args);
+        assert (!str);
+
+        str = zsys_zprintf_error ("VALUE=%(nokey)s123, %(ham)s, %(ham)s, %%(nospam)s!!!", args);
+        assert (streq (str, "Key 'nokey' not found in hash"));
+        zstr_free (&str);
+
+        str = zsys_zprintf ("VALUE=%(key)s/%%S", args);
+        assert (streq (str, "VALUE=value/%S"));
+        zstr_free (&str);
+
+        zhash_destroy (&args);
+
+        //ZPL based printf
+        zconfig_t *root = zconfig_new ("root", NULL);
+        zconfig_put (root, "zsp", "");
+        zconfig_put (root, "zsp/return_code", "0");
+
+        str = zsys_zplprintf ("return_code=%(zsp/return_code)s", root);
+        assert (streq (str, "return_code=0"));
+        zstr_free (&str);
+
+        zconfig_destroy (&root);
+    }
+
     //  @end
 
     zsys_set_auto_use_fd (1);
